#ifndef  __TET_H__
#define  __TET_H__

#include "tetconfig.h"

#ifdef X11
typedef unsigned long COLOUR_REF;
#else
typedef unsigned char COLOUR_REF; 
#endif 

#define ITET_VERSION 0x10603

#define FIG_WIDTH   16
#define FIG_HEIGHT  16
#define FGB_WIDTH  FIG_WIDTH/2
#define FGR_WIDTH  FGB_WIDTH
#define FGT_WIDTH  FIG_WIDTH/8

#define BRD_WIDTH    10         /* Cells per row */
#define BRD_HEIGHT   20         /* Cells per column */

#define FIG_DIM   4
#define NPATS   12
#define NFIGS  7
#define BOARD  char
#define PATHEIGHT 2
typedef char FIGURE[FIG_DIM][FIG_DIM];

#define MODE_REPEAT 1
#define MODE_FORCE  2
#define MODE_CHECK  4

#define get_rand(x) ((int)((double)rand() * (x) / RAND_MAX))

/* This works if x is a power of two */
#define get_rand1(x) (rand() & (x-1))

#define X11_CHIPSET  0x6666             

typedef struct
{       int left;
        int top;
        int right;
        int bottom;
} RECT;

typedef enum {W_PROLOGUE, W_BETWEEN, W_RESULTS, W_ADVICE, W_GAME} WHERE_WE_ARE;
typedef enum {B_SCROLLER=0, B_SHADOW=0, B_HELP=1, B_NONE=2} BOTTOM_MODE;


/* tetmain.c */
void SuspendTimer(void);
void ResumeTimer(void);
int ProceedSystemEvents(void);
void TerminationHandler(int code);
void ConfinePointer(void);
void ReleasePointer(void);

/* tetgame.c */
int move_figure(void);
int move_demo_figure(void);
void NewGame(void);
int EndGame(int par);


/* tetXdraw.c */
void InitBoard(void);
void DeinitBoard(void);
void DrawFramedRect(RECT *rc);
void DrawSolidRect(RECT *rc, int colour);
void EncompassFramedRect(RECT *rc);
void DrawPatRect(RECT *rc, char pattern[8], int offset);
void DrawBkgrPattern(RECT *rc);
void DrawAll(BOARD brd[][BRD_WIDTH]);
void DrawSolidLine(int x, int y, int x2, int y2,
                         int width, int Colour);
void DrawPatLine(int x, int y, int width, int pat, int fgr, int bkgr);

#ifndef X11
void setrow(int x, int y, int wid, unsigned char *row);
#endif

int  AllocScreenContext(void);
int  ScreenContextToScreen(void);
int  ContextRectToScreen(RECT *rc);
int  ScreenToScreenContext(void);
int  InvertScreenContext(void);
void  DeleteScreenContext(void);
void RefreshAndWait(long Int, RECT *rc);
void ProceedExposures(void);

//int  SetStdPalette(void);
int  InvertPalette(void);

/* tetfigure.c */
void get_figure(FIGURE figure, int type, int rot);
void cop_figure(FIGURE tofig, FIGURE fromfig); 
void embed_figure(BOARD brd[][BRD_WIDTH], FIGURE figure,
                            int x, int y, int wid, int hei);
int calc_cost(FIGURE figure, int x, int y, int wid, int hei);
int test_posit( FIGURE figure, int x,  int y, int wid, int hei);
void rot_left(FIGURE figure, FIGURE nfig);
void rot_right(FIGURE figure, FIGURE nfig);
int fig_width(FIGURE figure);
int fig_height(FIGURE figure);
void ShowShadowFrame(void);
void rot_left(FIGURE figure, FIGURE nfig);


/* tetoper.c */
void debug(char *message);
void PrepareStatWindow(void);
void PrepareJoyWindow(void);
void PrepareNextWindow(void);
int PrepareMusic(int Param);
void PrepareScroller(void);
void change_bcol(void);
void FillLayers(int Layers);
void PrepareShowGrid(FIGURE figure, int x, int y, int wid, int hei);
void ChangeLevel(FIGURE figure, int x, int y, int wid, int hei);
void toggle_sound(void);
void demo_plus(void);
void SetLevel(void);
void ShowBonus(void);
void ShowNextFigure(void);
void UpdateStatus(void);
void DrawEndGameMessage(void);
void HideBonus(void);
void InitHelpPages(void);
void MaintainHelp(void);
void ChangeHelpPage(void);
void ToggleBottomMode(void);
int  EnableJoystick(void);
void DisableJoystick(void);
char *GetPlayerName(void);

/* tetapropo.h */
int ShowStatistics(void);
int ShowIntergameScreen(void);
int InitIntergameScreen(void);
int ShowPrologue(void);

/* tetimage */
void drawpic(void);
void adjusteye(int x, int y);
void owlwave(int status);
void robthink(int status);
void drawrobbrd(char brd[][BRD_WIDTH], int x1, int x2, int y1, int y2);
int  InitRobImage(void);
void DeinitRobImage(void);


/* tetmisc.c */
int InitScroller(void);
int StartScroller(char *text, int speed, int randomN,  char *randomText[]);
void ShowScrollerFrame(void);
int DeinitScroller(void);
void ScrollerTick(void);
int IsScrollerActive(void);
void SuspendScroller(void);
void ResumeScroller(void);
void StopScroller(void);
int SetScrollerParam(int NewWidth);
int InitPlayLines(void);
void DeinitPlayLines(void);
int PlayPlayLine(int index, int mode);
char *GetFullFileName(const char *shortname);
int  StopBGPlay(int wait);
void ShowComHelp();
int SetColour(int  entry, char *ColourName);
void  process_modes(char *modes);
void sleep_until(long StopAt);

/* tetXcolours.c*/
typedef struct
{  unsigned char r, g, b; }  PAL_ENTRY;

typedef struct
{    unsigned short  imwid, imhei, imbpp;  // width, height, bit counter
     unsigned short  usedcol;              // used colours
     unsigned char   *image;
     PAL_ENTRY       *pal;
     COLOUR_REF      *PalEqu;
} IMAGE_HEADER;

enum TextPos
{   ScoreTextPos, LevelTextPos,
    LayersTextPos, BonusTextPos,
    NextTextPos, JoyTextPos,
    TotalTextPos, TopHeaderPos 
};

#define VAR_TEXTS   8

unsigned long AllocColour(char *ColourName);
int initPalette(void);
void deinitPalette(void);
void setPalette(IMAGE_HEADER *imHdr, int maxcols);
int  showImage(IMAGE_HEADER *imHdr, int x, int y, int transp);
int  showSqueezedImage(IMAGE_HEADER *imHdr, int x, int y, int transp,
                                               int stepx, int stepy);

/* tetXboard.c */					       
int  InitPixmapPats(void);
void FreePixmapPats(void);
void DrawBoard(BOARD brd[][BRD_WIDTH], int fromx, int tox, int fromy, int toy);
void DrawBoardRow(char brdrow[BRD_WIDTH], int fx, int tx, int y, unsigned char mask);
void ShowFigure(FIGURE figure, int px, int py, int tox, int toy);
void ShowShadow(FIGURE figure, int pos);
void ShowJoyPos(int active);
void PauseGame(void);
int ResumeGame(char *DispName, char *message, int flag);
void SuspendGame(void);
long GetWindowState(void);


/* tetXprepare.c */					       
extern int prepare_video(char *DispName); 
extern void terminate_video(void);

					       
/* tettext.c */
int getDemTextSize(void);

#endif
